@@ -8,22 +8,22 @@
 /*****************************************************************************/
 /*  MODULE IMPORTS:                                                          */
 /*****************************************************************************/
+#include <stdio.h>                                  /*  MODULE TYPE:  (sys)  */
 #include <stdlib.h>                                 /*  MODULE TYPE:  (sys)  */
 #include <string.h>                                 /*  MODULE TYPE:  (sys)  */
-#include <ctype.h>                                  /*  MODULE TYPE:  (sys)  */
 #include <time.h>                                   /*  MODULE TYPE:  (sys)  */
 #include "ToolBox.h"                                /*  MODULE TYPE:  (dat)  */
-
-#ifdef MACOS_TRADITIONAL
-  #ifndef __MACTYPES__
-    #include <MacTypes.h>
-  #endif
-#endif
-
 /*****************************************************************************/
 /*  MODULE INTERFACE:                                                        */
 /*****************************************************************************/
 
+/* Make the VMS linker happy: */
+
+#ifdef VMS
+#define DateCalc_Day_of_Week_Abbreviation_ DateCalc_DoW_Abbrev_
+#define DateCalc_nth_weekday_of_month_year DateCalc_nth_weekday
+#endif
+
 boolean
 DateCalc_leap_year                     (Z_int   year);
 
@@ -32,6 +32,11 @@ DateCalc_check_date                    (Z_int   year,
                                         Z_int   month,
                                         Z_int   day);
 
+boolean
+DateCalc_check_time                    (Z_int   hour,
+                                        Z_int   min,
+                                        Z_int   sec);
+
 boolean
 DateCalc_check_business_date           (Z_int   year,
                                         Z_int   week,
@@ -101,6 +106,18 @@ DateCalc_Delta_Days                    (Z_int   year1,
                                         Z_int   month2,
                                         Z_int   day2);
 
+boolean /* PRIVATE */
+DateCalc_delta_hms                     (Z_long *Dd,         /*  I/O  */
+                                        Z_int  *Dh,         /*   O   */
+                                        Z_int  *Dm,         /*   O   */
+                                        Z_int  *Ds,         /*   O   */
+                                        Z_int   hour1,      /*   I   */
+                                        Z_int   min1,       /*   I   */
+                                        Z_int   sec1,       /*   I   */
+                                        Z_int   hour2,      /*   I   */
+                                        Z_int   min2,       /*   I   */
+                                        Z_int   sec2);      /*   I   */
+
 boolean
 DateCalc_delta_dhms                    (Z_long *Dd,         /*   O   */
                                         Z_int  *Dh,         /*   O   */
@@ -119,6 +136,40 @@ DateCalc_delta_dhms                    (Z_long *Dd,         /*   O   */
                                         Z_int   min2,       /*   I   */
                                         Z_int   sec2);      /*   I   */
 
+boolean
+DateCalc_delta_ymd                     (Z_int  *year1,      /*  I/O  */
+                                        Z_int  *month1,     /*  I/O  */
+                                        Z_int  *day1,       /*  I/O  */
+                                        Z_int   year2,      /*   I   */
+                                        Z_int   month2,     /*   I   */
+                                        Z_int   day2);      /*   I   */
+
+boolean
+DateCalc_delta_ymdhms                  (Z_int  *D_y,        /*   O   */
+                                        Z_int  *D_m,        /*   O   */
+                                        Z_int  *D_d,        /*   O   */
+                                        Z_int  *Dh,         /*   O   */
+                                        Z_int  *Dm,         /*   O   */
+                                        Z_int  *Ds,         /*   O   */
+                                        Z_int   year1,      /*   I   */
+                                        Z_int   month1,     /*   I   */
+                                        Z_int   day1,       /*   I   */
+                                        Z_int   hour1,      /*   I   */
+                                        Z_int   min1,       /*   I   */
+                                        Z_int   sec1,       /*   I   */
+                                        Z_int   year2,      /*   I   */
+                                        Z_int   month2,     /*   I   */
+                                        Z_int   day2,       /*   I   */
+                                        Z_int   hour2,      /*   I   */
+                                        Z_int   min2,       /*   I   */
+                                        Z_int   sec2);      /*   I   */
+
+void
+DateCalc_Normalize_DHMS                (Z_long *Dd,         /*  I/O  */
+                                        Z_long *Dh,         /*  I/O  */
+                                        Z_long *Dm,         /*  I/O  */
+                                        Z_long *Ds);        /*  I/O  */
+
 boolean
 DateCalc_add_delta_days                (Z_int  *year,       /*  I/O  */
                                         Z_int  *month,      /*  I/O  */
@@ -137,6 +188,19 @@ DateCalc_add_delta_dhms                (Z_int  *year,       /*  I/O  */
                                         Z_long  Dm,         /*   I   */
                                         Z_long  Ds);        /*   I   */
 
+boolean /* PRIVATE */
+DateCalc_add_year_month                (Z_int  *year,       /*  I/O  */
+                                        Z_int  *month,      /*  I/O  */
+                                        Z_long  Dy,         /*   I   */
+                                        Z_long  Dm);        /*   I   */
+
+boolean
+DateCalc_add_delta_ym                  (Z_int  *year,       /*  I/O  */
+                                        Z_int  *month,      /*  I/O  */
+                                        Z_int  *day,        /*  I/O  */
+                                        Z_long  Dy,         /*   I   */
+                                        Z_long  Dm);        /*   I   */
+
 boolean
 DateCalc_add_delta_ymd                 (Z_int  *year,       /*  I/O  */
                                         Z_int  *month,      /*  I/O  */
@@ -145,6 +209,20 @@ DateCalc_add_delta_ymd                 (Z_int  *year,       /*  I/O  */
                                         Z_long  Dm,         /*   I   */
                                         Z_long  Dd);        /*   I   */
 
+boolean
+DateCalc_add_delta_ymdhms              (Z_int  *year,       /*  I/O  */
+                                        Z_int  *month,      /*  I/O  */
+                                        Z_int  *day,        /*  I/O  */
+                                        Z_int  *hour,       /*  I/O  */
+                                        Z_int  *min,        /*  I/O  */
+                                        Z_int  *sec,        /*  I/O  */
+                                        Z_long  D_y,        /*   I   */
+                                        Z_long  D_m,        /*   I   */
+                                        Z_long  D_d,        /*   I   */
+                                        Z_long  Dh,         /*   I   */
+                                        Z_long  Dm,         /*   I   */
+                                        Z_long  Ds);        /*   I   */
+
 boolean
 DateCalc_system_clock                  (Z_int  *year,       /*   O   */
                                         Z_int  *month,      /*   O   */
@@ -154,7 +232,72 @@ DateCalc_system_clock                  (Z_int  *year,       /*   O   */
                                         Z_int  *sec,        /*   O   */
                                         Z_int  *doy,        /*   O   */
                                         Z_int  *dow,        /*   O   */
-                                        Z_int  *dst);       /*   O   */
+                                        Z_int  *dst,        /*   O   */
+                                        boolean gmt);       /*   I   */
+
+boolean
+DateCalc_gmtime                        (Z_int  *year,       /*   O   */
+                                        Z_int  *month,      /*   O   */
+                                        Z_int  *day,        /*   O   */
+                                        Z_int  *hour,       /*   O   */
+                                        Z_int  *min,        /*   O   */
+                                        Z_int  *sec,        /*   O   */
+                                        Z_int  *doy,        /*   O   */
+                                        Z_int  *dow,        /*   O   */
+                                        Z_int  *dst,        /*   O   */
+                                        time_t  seconds);   /*   I   */
+
+boolean
+DateCalc_localtime                     (Z_int  *year,       /*   O   */
+                                        Z_int  *month,      /*   O   */
+                                        Z_int  *day,        /*   O   */
+                                        Z_int  *hour,       /*   O   */
+                                        Z_int  *min,        /*   O   */
+                                        Z_int  *sec,        /*   O   */
+                                        Z_int  *doy,        /*   O   */
+                                        Z_int  *dow,        /*   O   */
+                                        Z_int  *dst,        /*   O   */
+                                        time_t  seconds);   /*   I   */
+
+boolean
+DateCalc_mktime                        (time_t *seconds,    /*   O   */
+                                        Z_int   year,       /*   I   */
+                                        Z_int   month,      /*   I   */
+                                        Z_int   day,        /*   I   */
+                                        Z_int   hour,       /*   I   */
+                                        Z_int   min,        /*   I   */
+                                        Z_int   sec,        /*   I   */
+                                        Z_int   doy,        /*   I   */
+                                        Z_int   dow,        /*   I   */
+                                        Z_int   dst);       /*   I   */
+
+boolean
+DateCalc_timezone                      (Z_int  *year,       /*   O   */
+                                        Z_int  *month,      /*   O   */
+                                        Z_int  *day,        /*   O   */
+                                        Z_int  *hour,       /*   O   */
+                                        Z_int  *min,        /*   O   */
+                                        Z_int  *sec,        /*   O   */
+                                        Z_int  *dst,        /*   O   */
+                                        time_t when);       /*   I   */
+
+boolean
+DateCalc_date2time                     (time_t *seconds,    /*   O   */
+                                        Z_int   year,       /*   I   */
+                                        Z_int   month,      /*   I   */
+                                        Z_int   day,        /*   I   */
+                                        Z_int   hour,       /*   I   */
+                                        Z_int   min,        /*   I   */
+                                        Z_int   sec);       /*   I   */
+
+boolean
+DateCalc_time2date                     (Z_int  *year,       /*   O   */
+                                        Z_int  *month,      /*   O   */
+                                        Z_int  *day,        /*   O   */
+                                        Z_int  *hour,       /*   O   */
+                                        Z_int  *min,        /*   O   */
+                                        Z_int  *sec,        /*   O   */
+                                        time_t  seconds);   /*   I   */
 
 boolean
 DateCalc_easter_sunday                 (Z_int  *year,       /*  I/O  */
@@ -185,6 +328,12 @@ DateCalc_decode_date_us                (charptr buffer,     /*   I   */
                                         Z_int  *month,      /*   O   */
                                         Z_int  *day);       /*   O   */
 
+Z_int
+DateCalc_Fixed_Window                  (Z_int   year);
+
+Z_int
+DateCalc_Moving_Window                 (Z_int   year);
+
 Z_int
 DateCalc_Compress                      (Z_int   year,
                                         Z_int   month,
@@ -219,11 +368,18 @@ DateCalc_English_Ordinal               (charptr result,     /*   O   */
 
 charptr
 DateCalc_Calendar                      (Z_int   year,
-                                        Z_int   month);
+                                        Z_int   month,
+                                        boolean orthodox);
 
 void
 DateCalc_Dispose                       (charptr string);
 
+N_char
+DateCalc_ISO_LC                        (N_char c);
+
+N_char
+DateCalc_ISO_UC                        (N_char c);
+
 charptr
 DateCalc_Version                       (void);
 
@@ -251,7 +407,7 @@ extern const Z_int DateCalc_Days_in_Month_[2][13];
 };
 */
 
-#define DateCalc_LANGUAGES 7
+#define DateCalc_LANGUAGES 13
 
 extern Z_int  DateCalc_Language; /* Default = 1 (English) */
 
@@ -267,28 +423,53 @@ extern const N_char DateCalc_Month_to_Text_[DateCalc_LANGUAGES+1][13][32];
         "July", "August", "September", "October", "November", "December"
     },
     {
-        "???", "Janvier", "Fvrier", "Mars", "Avril", "Mai", "Juin",
-        "Juillet", "Aot", "Septembre", "Octobre", "Novembre", "Dcembre"
+        "???", "janvier", "fvrier", "mars", "avril", "mai", "juin",
+        "juillet", "aot", "septembre", "octobre", "novembre", "dcembre"
     },
     {
         "???", "Januar", "Februar", "Mrz", "April", "Mai", "Juni",
         "Juli", "August", "September", "Oktober", "November", "Dezember"
     },
     {
-        "???", "Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio",
-        "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre"
+        "???", "enero", "febrero", "marzo", "abril", "mayo", "junio",
+        "julio", "agosto", "septiembre", "octubre", "noviembre", "diciembre"
     },
     {
-        "???", "Janeiro", "Fevereiro", "Maro", "Abril", "Maio", "Junho",
-        "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"
+        "???", "janeiro", "fevereiro", "maro", "abril", "maio", "junho",
+        "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"
     },
     {
-        "???", "Januari", "Februari", "Maart", "April", "Mei", "Juni",
-        "Juli", "Augustus", "September", "October", "November", "December"
+        "???", "januari", "februari", "maart", "april", "mei", "juni",
+        "juli", "augustus", "september", "october", "november", "december"
     },
     {
         "???", "Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno",
         "Luglio", "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre"
+    },
+    {
+        "???", "januar", "februar", "mars", "april", "mai", "juni",
+        "juli", "august", "september", "oktober", "november", "desember"
+    },
+    {
+        "???", "januari", "februari", "mars", "april", "maj", "juni",
+        "juli", "augusti", "september", "oktober", "november", "december"
+    },
+    {
+        "???", "januar", "februar", "marts", "april", "maj", "juni",
+        "juli", "august", "september", "oktober", "november", "december"
+    },
+    {
+        "???", "tammikuu", "helmikuu", "maaliskuu", "huhtikuu",
+        "toukokuu", "keskuu", "heinkuu", "elokuu",
+        "syyskuu", "lokakuu", "marraskuu", "joulukuu"
+    },
+    {
+        "???", "Janur", "Februr", "Mrcius", "prilis", "Mjus", "Jnius",
+        "Jlius", "Augusztus", "Szeptember", "Oktber", "November", "December"
+    },
+    {
+        "???", "Styczen", "Luty", "Marzec", "Kwiecien", "Maj", "Czerwiec",
+        "Lipiec", "Sierpien", "Wrzesien", "Pazdziernik", "Listopad", "Grudzien"
     }
 };
 */
@@ -327,6 +508,30 @@ extern const N_char DateCalc_Day_of_Week_to_Text_[DateCalc_LANGUAGES+1][8][32];
     {
         "???", "Luned", "Marted", "Mercoled",
         "Gioved", "Venerd", "Sabato", "Domenica"
+    },
+    {
+        "???", "mandag", "tirsdag", "onsdag",
+        "torsdag", "fredag", "lrdag", "sndag"
+    },
+    {
+        "???", "mndag", "tisdag", "onsdag",
+        "torsdag", "fredag", "lrdag", "sndag"
+    },
+    {
+        "???", "mandag", "tirsdag", "onsdag",
+        "torsdag", "fredag", "lrdag", "sndag"
+    },
+    {
+        "???", "maanantai", "tiistai", "keskiviikko",
+        "torstai", "perjantai", "lauantai", "sunnuntai"
+    },
+    {
+        "???", "htf", "kedd", "szerda",
+        "cstrtk", "pntek", "szombat", "vasrnap"
+    },
+    {
+        "???", "Poniedzialek", "Wtorek", "Sroda",
+        "Czwartek", "Piatek", "Sobota", "Niedziela"
     }
 };
 */
@@ -360,6 +565,24 @@ extern const N_char DateCalc_Day_of_Week_Abbreviation_[DateCalc_LANGUAGES+1][8][
     },
     {
         "", "", "", "", "", "", "", ""
+    },
+    {
+        "", "", "", "", "", "", "", ""
+    },
+    {
+        "", "", "", "", "", "", "", ""
+    },
+    {
+        "", "", "", "", "", "", "", ""
+    },
+    {
+        "", "", "", "", "", "", "", ""
+    },
+    {
+        "", "", "", "", "", "", "", ""
+    },
+    {
+        "???", "Pn", "Wt", "Sr", "Cz", "Pt", "So", "Ni"
     }
 };
 */
@@ -379,11 +602,17 @@ extern const N_char DateCalc_Date_Long_Format_[DateCalc_LANGUAGES+1][64];
 {
     "%s, %d %s %d",
     "%s, %s %s %d",
-    "%s, le %d %s %d",
+    "%s %d %s %d",
     "%s, den %d. %s %d",
     "%s, %d de %s de %d",
     "%s, dia %d de %s de %d",
+    "%s, %d %s %d",
+    "%s, %d %s %d",
     "%s, %d. %s %d",
+    "%s, %d %s %d",
+    "%s, %d. %s %d",
+    "%s, %d. %sta %d",
+    "%d. %s %d., %s",
     "%s, %d %s %d"
 };
 */
@@ -392,7 +621,8 @@ extern const N_char DateCalc_Language_to_Text_[DateCalc_LANGUAGES+1][32];
 /*
 {
     "???", "English", "Franais", "Deutsch", "Espaol",
-    "Portugus", "Nederlands", "Italiano"
+    "Portugus", "Nederlands", "Italiano", "Norsk", "Svenska",
+    "Dansk", "suomi", "Magyar", "Polski"
 };
 */
 
@@ -401,11 +631,14 @@ extern const N_char DateCalc_Language_to_Text_[DateCalc_LANGUAGES+1][32];
 /*****************************************************************************/
 
 /*****************************************************************************/
-/*  VERSION:  4.3                                                            */
+/*  VERSION:  5.2                                                            */
 /*****************************************************************************/
 /*  VERSION HISTORY:                                                         */
 /*****************************************************************************/
 /*                                                                           */
+/*    Version 5.2   18.09.02  No changes.                                    */
+/*    Version 5.1   08.09.02  Added conditional changes for MacOS/MacPerl.   */
+/*    Version 5.0   10.10.01  New YMD/HMS functions, replaced <ctype.h>, ... */
 /*    Version 4.3   08.01.00  decode_date_??: (yy < 70 ? 20yy : 19yy)        */
 /*    Version 4.2   07.09.98  No changes.                                    */
 /*    Version 4.1   08.06.98  Fixed bug in "add_delta_ymd()".                */
@@ -433,10 +666,6 @@ extern const N_char DateCalc_Language_to_Text_[DateCalc_LANGUAGES+1][32];
 /*****************************************************************************/
 /*                                                                           */
 /*    Steffen Beyer                                                          */
-/*    Ainmillerstr. 5 / App. 513                                             */
-/*    D-80801 Munich                                                         */
-/*    Germany                                                                */
-/*                                                                           */
 /*    mailto:sb@engelschall.com                                              */
 /*    http://www.engelschall.com/u/sb/download/                              */
 /*                                                                           */
@@ -444,7 +673,7 @@ extern const N_char DateCalc_Language_to_Text_[DateCalc_LANGUAGES+1][32];
 /*  COPYRIGHT:                                                               */
 /*****************************************************************************/
 /*                                                                           */
-/*    Copyright (c) 1993 - 2000 by Steffen Beyer.                            */
+/*    Copyright (c) 1993 - 2002 by Steffen Beyer.                            */
 /*    All rights reserved.                                                   */
 /*                                                                           */
 /*****************************************************************************/
